"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var fs = _interopRequireWildcard(require("fs-extra"));

var path = _interopRequireWildcard(require("path"));

var _globby = _interopRequireDefault(require("globby"));

var _changeCase = require("change-case");

var _prompts = _interopRequireDefault(require("prompts"));

var _getModularRoot = _interopRequireDefault(require("./utils/getModularRoot"));

var _execAsync = _interopRequireDefault(require("./utils/execAsync"));

var logger = _interopRequireWildcard(require("./utils/logger"));

var _actionPreflightCheck = _interopRequireDefault(require("./utils/actionPreflightCheck"));

var _getAllFiles = _interopRequireDefault(require("./utils/getAllFiles"));

var _LineFilterOutStream = _interopRequireDefault(require("./utils/LineFilterOutStream"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

const packagesRoot = 'packages';
const CUSTOM_TEMPLATE = '__CHOOSE_MY_OWN__';

async function promptForName(name) {
  // The user may try to create a package using a name that already exists
  // so we loop until they choose a unique name or cancel
  while (true) {
    // Default the response the predefined `name` argument
    const response = name ? {
      name
    } : await (0, _prompts.default)({
      name: 'name',
      type: 'text',
      message: `What would you like to name this package?`
    }); // Exit if the user cancels

    if (!response.name) {
      throw Error('No name entered, exiting');
    } // Check whether the package exists already


    const modularRoot = (0, _getModularRoot.default)();
    const packagePath = path.join(modularRoot, packagesRoot, response.name);

    if (fs.existsSync(packagePath)) {
      logger.error(`A package called "${response.name}" already exists!`);
      name = undefined;
    } else {
      return response.name;
    }
  }
}

async function promptForType(type) {
  if (type) {
    return type;
  }

  const response = await (0, _prompts.default)({
    name: 'type',
    type: 'select',
    message: `What kind of package is this?`,
    choices: [{
      title: 'A plain package (package)',
      value: 'package'
    }, {
      title: 'A micro-frontend React view compiled to a bundle file (view)',
      value: 'view'
    }, {
      title: 'A micro-frontend React view compiled to ES Modules (esm-view)',
      value: 'esm-view'
    }, {
      title: 'A micro-frontend React container (app)',
      value: 'app'
    }, {
      title: 'Choose my own template',
      value: CUSTOM_TEMPLATE
    }],
    initial: 0
  });

  if (!response.type) {
    throw Error('No type entered, exiting');
  }

  return response.type;
}

async function promptForTemplate(templateName) {
  if (templateName === CUSTOM_TEMPLATE) {
    logger.warn('Note: you are choosing to install a template that is not maintained by the modular team.');
    const response = await (0, _prompts.default)([{
      name: 'templateName',
      type: 'text',
      message: `Which modular template would you like to use?`
    }]);

    if (!response.templateName) {
      throw Error('No template name entered, exiting');
    }

    if (response.templateName[0] === '@') {
      const [scope, typePackageName] = response.templateName.split('/');
      templateName = `${scope}/modular-template-${typePackageName}`;
    } else {
      templateName = response.templateName;
    }
  }

  if (!templateName.startsWith('modular-template')) {
    return `modular-template-${templateName}`;
  }

  return templateName;
}

async function getYarnVersion() {
  const {
    stdout: version
  } = await (0, _execAsync.default)('yarnpkg', ['--version'], {
    cwd: (0, _getModularRoot.default)(),
    stdout: 'pipe'
  });
  return version;
}

async function addPackage({
  name: nameArg,
  type: typeArg,
  template: templateNameArg,
  preferOffline = true,
  verbose = false,
  unstableName
}) {
  var _modularTemplatePacka, _modularTemplatePacka2, _subprocess$stderr;

  const name = await promptForName(nameArg || unstableName);
  const packageType = templateNameArg ?? (await promptForType(typeArg));
  const templateName = await promptForTemplate(templateNameArg || packageType);
  const modularRoot = (0, _getModularRoot.default)();
  const yarnVersion = await getYarnVersion();
  const isYarnV1 = yarnVersion.startsWith('1.');
  const packageName = (0, _changeCase.paramCase)(name, {
    stripRegexp: /[^A-Z0-9@/]+/gi
  });
  const packageDir = (0, _changeCase.paramCase)(name, {
    stripRegexp: /[^A-Z0-9/]+/gi
  });
  const componentName = (0, _changeCase.pascalCase)(name);
  const packagePath = path.join(modularRoot, packagesRoot, packageDir);
  const installedPackageJsonPath = path.join(templateName, 'package.json');
  let newModularPackageJsonPath; // try and find the modular template packge, if it's already been installed
  // in the project then continue without needing to do an install.
  // else we will fetch it from the yarn registry.

  try {
    newModularPackageJsonPath = require.resolve(installedPackageJsonPath);
  } catch (e) {
    var _templateInstallSubpr;

    logger.log('Installing package template, this may take a moment...');
    const yarnAddArgs = ['add', templateName];

    if (isYarnV1) {
      yarnAddArgs.push('--prefer-offline', '-W');
    } else {
      yarnAddArgs.push('--cached');
    }

    const templateInstallSubprocess = (0, _execAsync.default)('yarnpkg', yarnAddArgs, {
      cwd: modularRoot,
      stderr: 'pipe',
      stdout: 'ignore'
    }); // Remove warnings

    (_templateInstallSubpr = templateInstallSubprocess.stderr) === null || _templateInstallSubpr === void 0 ? void 0 : _templateInstallSubpr.pipe(new _LineFilterOutStream.default(/.*warning.*/));

    if (verbose) {
      var _templateInstallSubpr2;

      (_templateInstallSubpr2 = templateInstallSubprocess.stderr) === null || _templateInstallSubpr2 === void 0 ? void 0 : _templateInstallSubpr2.pipe(process.stderr);
    }

    await templateInstallSubprocess;
    newModularPackageJsonPath = require.resolve(installedPackageJsonPath);
  }

  const modularTemplatePackageJson = await fs.readJSON(newModularPackageJsonPath);
  const modularType = modularTemplatePackageJson === null || modularTemplatePackageJson === void 0 ? void 0 : (_modularTemplatePacka = modularTemplatePackageJson.modular) === null || _modularTemplatePacka === void 0 ? void 0 : _modularTemplatePacka.type;

  if (modularType !== 'template') {
    throw new Error(`${templateName} is not marked as a "template". Check the package you are trying to install.`);
  }

  const modularTemplateType = modularTemplatePackageJson === null || modularTemplatePackageJson === void 0 ? void 0 : (_modularTemplatePacka2 = modularTemplatePackageJson.modular) === null || _modularTemplatePacka2 === void 0 ? void 0 : _modularTemplatePacka2.templateType;

  if (!['app', 'esm-view', 'view', 'package'].includes(modularTemplateType)) {
    throw new Error(`${templateName} has modular type: ${modularTemplateType}, which does not exist, please use update this template`);
  }

  const packageTypePath = path.dirname(newModularPackageJsonPath); // create a new package source folder

  await fs.mkdirp(packagePath);
  await fs.copy(packageTypePath, packagePath, {
    recursive: true,

    filter(src) {
      return !(path.basename(src) === 'package.json');
    }

  });
  const packageFilePaths = (0, _getAllFiles.default)(packagePath); // If we get our package locally we need to whitelist files like yarn publish does

  const packageWhitelist = _globby.default.sync(modularTemplatePackageJson.files || ['*'], {
    cwd: packagePath,
    absolute: true
  }).map(filePath => path.normalize(filePath));

  for (const packageFilePath of packageFilePaths) {
    if (!packageWhitelist.includes(packageFilePath)) {
      fs.removeSync(packageFilePath);
    } else if (/\.(ts|tsx|js|jsx|json|md|txt)$/i.test(packageFilePath)) {
      fs.writeFileSync(packageFilePath, fs.readFileSync(packageFilePath, 'utf8').replace(/PackageName__/g, packageName).replace(/ComponentName__/g, componentName));
    }
  }

  await fs.writeJson(path.join(packagePath, 'package.json'), {
    name: packageName,
    private: modularTemplateType === 'app',
    modular: {
      type: modularTemplateType
    },
    main: modularTemplatePackageJson === null || modularTemplatePackageJson === void 0 ? void 0 : modularTemplatePackageJson.main,
    dependencies: modularTemplatePackageJson === null || modularTemplatePackageJson === void 0 ? void 0 : modularTemplatePackageJson.dependencies,
    version: '1.0.0'
  }, {
    spaces: 2
  });

  if (modularTemplateType === 'app') {
    // add a tsconfig, because CRA expects it
    await fs.writeJSON(path.join(packagePath, 'tsconfig.json'), {
      extends: path.relative(packagePath, modularRoot) + '/tsconfig.json'
    }, {
      spaces: 2
    });
  }

  const yarnArgs = [];

  if (isYarnV1) {
    if (verbose) {
      yarnArgs.push('--verbose');
    }

    if (preferOffline) {
      yarnArgs.push('--prefer-offline');
    }
  }

  const subprocess = (0, _execAsync.default)('yarnpkg', yarnArgs, {
    cwd: modularRoot,
    stderr: 'pipe',
    stdout: verbose ? process.stdout : 'ignore'
  }); // Remove warnings

  (_subprocess$stderr = subprocess.stderr) === null || _subprocess$stderr === void 0 ? void 0 : _subprocess$stderr.pipe(new _LineFilterOutStream.default(/.*warning.*/));

  if (verbose) {
    var _subprocess$stderr2;

    (_subprocess$stderr2 = subprocess.stderr) === null || _subprocess$stderr2 === void 0 ? void 0 : _subprocess$stderr2.pipe(process.stderr);
  }

  await subprocess;
}

var _default = (0, _actionPreflightCheck.default)(addPackage);

exports.default = _default;
//# sourceMappingURL=addPackage.js.map